<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 19.12.2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 *
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Zend/Config.php';
require_once 'Hpls/Utils.php';
require_once 'Hpls/Job/Step.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package AC
 * @subpackage Job-Queue
 * @version $LastChangedRevision$
 * @since 1.0.7
 */
abstract class Hpls_Job_StatusAbstract {
	
	protected $_jobName = '';
	protected $_jobID = null;
	protected $_jobHost = null;
	/**
	 * Um einen Job vorzeitig als Beendet zu erklären, obwohl noch nicht alle Step abgeschlossen sind. (z.B. bei failStep())
	 * @var unknown_type
	 */
	protected $_jobFinished = false;
	
	protected $_loaded = false;
	
	protected $_steps = array();
	protected $_currentStep = -1;
	protected $_lastStartStepID = null;

	/**
	 * Enter description here ...
	 * @param string|Hpls_Job $jobName
	 * @param string|null $jobID
	 * @param string $jobHost
	 */
	public function __construct($jobName, $jobID, $jobHost = 'localhost') {
		if ($jobName instanceof Hpls_Job_Abstract) {
			$this->_jobHost = $jobName->getJobName();
			$this->_jobID = $jobName->getJobID();
			$this->_jobName = $jobName->getJobName();
		} else {
			$this->_jobName = $jobName;
			$this->_jobID = $jobID;
			$this->_jobHost = $jobHost;
		}
	}

	/**
	 * Fügt einen neuen Abarbeitungsschritt dem Status hinzu und erzeugt dabei die nötige Datenstruktur.
	 * <p>Sollte nicht mehr gerufen werden, nachdem mit load() ein persistierter Status gelesen wurde.
	 * @param unknown_type $stepID
	 */
	public function addStep($stepID) {
		$stepOrder = $this->getNextFreeStepID();
		$step = null;
		if ($stepID instanceof Hpls_Job_Step) {
			$step = $stepID;
			$step->setOwner($this);
		} elseif (is_array($stepID)) {
			$step = new Hpls_Job_Step($this, $stepID, $order);
		} else {
			$step = new Hpls_Job_Step($this, $stepID, $stepOrder);
		}
		$this->_steps[$stepID] = $step;
	}

	public function removeStep($stepID) {
		unset($this->_steps[$stepID]);
	}

	public function startStep($stepID, $percentAtStart = 0) {
		$step = &$this->_steps[$stepID];
		if (is_null($step)) {
			require_once 'Hpls/Exception/Job/Status.php';
			throw new Hpls_Exception_Job_Status("StepID '$stepID' not defined.", 1);
		}
		$step->percent = (int) $percentAtStart;
		$step->state = Hpls_Job_Step::STATE_RUNNING;
		$step->starttime = time();
		$this->_lastStartStepID = $stepID;
		$this->save();
		Hpls_Utils::log()->info("Job '" . $this->_jobName . $this->_jobID . "': Step '$stepID' started.");
	}

	public function finishStep($stepID, $percentAtFinish = 100) {
		$step = &$this->_steps[$stepID];
		if (is_null($step)) {
			require_once 'Hpls/Exception/Job/Status.php';
			throw new Hpls_Exception_Job_Status("StepID '$stepID' not defined.", 1);
		}
		$step->percent = (int) $percentAtFinish;
		$step->state = Hpls_Job_Step::STATE_FINISHED;
		$step->stoptime = time();
		$this->save();
		Hpls_Utils::log()->info("Job '" . $this->_jobName . $this->_jobID . "': Step '$stepID' finished.");
	}

	/**
	 * Setzt einen Step als gescheitert und schreibt die Fehlermeldungen in den Status und ggf. die Exception.
	 * @param string $stepID
	 * @param string|array:string $errMessage
	 * @param Exception|null $exception
	 * @param boolean $setFinished
	 * @throws Hpls_Exception_Job_Status
	 */
	public function failStep($stepID, $errMessage, $exception = null, $setFinished = true) {
		$step = &$this->_steps[$stepID];
		if (is_null($step)) {
			require_once 'Hpls/Exception/Job/Status.php';
			throw new Hpls_Exception_Job_Status("StepID '$stepID' not defined.", 1);
		}
		if (!is_array($errMessage)) {
			$errMessage = array(
					$errMessage );
		}
		
		$step->percent = -1;
		$step->state = Hpls_Job_Step::STATE_FAILED;
		$step->errorMessages = $errMessage;
		//folgendes gibt fehler beim parsen der INI-datei wegen der Hashes '#' im stacktrace:
		//		if (!is_null($exception)) {
		//			$step->exception = serialize($exception);
		//		}
		$step->stoptime = time();
		$this->_jobFinished = $setFinished;
		$this->save();
		Hpls_Utils::log()->err("Job '" . $this->_jobName . $this->_jobID . "': Step '$stepID' FAILED. Message: " . implode("\n", $errMessage));
	}

	public function abortStep($stepID, $message = null, $setFinished = true) {
		$step = &$this->_steps[$stepID];
		if (is_null($step)) {
			require_once 'Hpls/Exception/Job/Status.php';
			throw new Hpls_Exception_Job_Status("StepID '$stepID' not defined.", 1);
		}
		if (!is_null($message)) {
			if (!is_array($message)) {
				$message = array(
						$message );
			}
			$step->messages = $message;
		}
		
		$step->percent = -1;
		$step->state = Hpls_Job_Step::STATE_ABORTED;
		$step->stoptime = time();
		$this->_jobFinished = $setFinished;
		$this->save();
		Hpls_Utils::log()->err("Job '" . $this->_jobName . $this->_jobID . "': Step '$stepID' ABORTED.");
	}

	public function setCurrentStep($stepID) {
		$step = &$this->_steps[$stepID];
		if (is_null($step)) {
			require_once 'Hpls/Exception/Job/Status.php';
			throw new Hpls_Exception_Job_Status("StepID '$stepID' not defined.", 1);
		}
		$this->_currentStep = $stepID;
	}

	public function progressStep($stepID, $percent, $absolute = false) {
		$step = &$this->_steps[$stepID];
		if (is_null($step)) {
			require_once 'Hpls/Exception/Job/Status.php';
			throw new Hpls_Exception_Job_Status("StepID '$stepID' not defined.", 1);
		}
		if ($absolute) {
			$step->percent = (int) $percent;
		} else {
			$step->percent += (int) $percent;
		}
		$this->save();
	}

	public function getProgress($stepID) {
		$step = &$this->_steps[$stepID];
		if (is_null($step)) {
			require_once 'Hpls/Exception/Job/Status.php';
			throw new Hpls_Exception_Job_Status("StepID '$stepID' not defined.", 1);
		}
		return $step->percent;
	}

	public function addStepMessage($stepID, $message) {
		$step = &$this->_steps[$stepID];
		if (is_null($step)) {
			require_once 'Hpls/Exception/Job/Status.php';
			throw new Hpls_Exception_Job_Status("StepID '$stepID' not defined.", 1);
		}
		$msgs = isset($step->messages) ? $step->messages : array();
		$msgs[] = $message;
		$step->messages = $msgs;
		$this->save();
	}

	public function setStepMessages($stepID, $messages) {
		$step = &$this->_steps[$stepID];
		if (is_null($step)) {
			require_once 'Hpls/Exception/Job/Status.php';
			throw new Hpls_Exception_Job_Status("StepID '$stepID' not defined.", 1);
		}
		if (!is_null($messages) && !is_array($messages)) {
			$messages = array(
					$messages );
		}
		$step->messages = $messages;
		$this->save();
	}

	public function getStepState($stepID) {
		$step = &$this->_steps[$stepID];
		if (is_null($step)) {
			require_once 'Hpls/Exception/Job/Status.php';
			throw new Hpls_Exception_Job_Status("StepID '$stepID' not defined.", 1);
		}
		return $step->state;
	}

	public function setJobFinished() {
		$this->_jobFinished = TRUE;
		$this->save();
	}

	public function isFinished() {
		if ($this->_jobFinished) {
			return true;
		}
		foreach ($this->_steps as $step) {
			if (!$step->isFinished()) {
				Hpls_Debug::log()->debug(__METHOD__ . " not FINISHED");
				return false;
			}
		}
		Hpls_Debug::log()->debug(__METHOD__ . " FINISHED");
		return true;
	}

	public function isAborted() {
		foreach ($this->_steps as $step) {
			if ($step->isAborted()) {
				Hpls_Debug::log()->debug(__METHOD__ . " ABORTED");
				return true;
			}
		}
		Hpls_Debug::log()->debug(__METHOD__ . " not ABORTED");
		return false;
	}

	public function isRunning() {
		foreach ($this->_steps as $step) {
			if ($step->isRunning()) {
				Hpls_Debug::log()->debug(__METHOD__ . " RUNNING");
				return true;
			}
		}
		Hpls_Debug::log()->debug(__METHOD__ . " not RUNNING");
		return false;
	}

	public function isStarted() {
		foreach ($this->_steps as $step) {
			if ($step->isStarted()) {
				Hpls_Debug::log()->debug(__METHOD__ . " STARTED");
				return true;
			}
		}
		Hpls_Debug::log()->debug(__METHOD__ . " not STARTED");
		return false;
	}

	public function getCurrentStep() {
		return $this->_currentStep;
	}

	public function &getStep($stepID) {
		if (isset($this->_steps[$stepID])) {
			
			return $this->_steps[$stepID];
		}
		return null;
	}

	public function getSteps() {
		return $this->_steps;
	}

	/**
	 * @return the $_jobName
	 */
	public function getJobName() {
		return $this->_jobName;
	}

	/**
	 * @return the $_jobID
	 */
	public function getJobID() {
		return $this->_jobID;
	}

	public function isStatusForJob(Hpls_Job_Abstract $job) {
		return ($this->_jobName == $job->getJobName()) && ($this->_jobID == $job->getJobID()) && ($this->_jobHost == $job->getJobHost());
	}

	/**
	 * Schreibt den Status in den Persistenzspeicher
	 * @throws Hpls_Exception_Job
	 */
	abstract public function save();

	/**
	 * Liest den Status aus dem Persistenzspeicher
	 * @throws Hpls_Exception_Job
	 */
	abstract public function load();

	abstract public function hasPersistedState();

	public function isLoaded() {
		return $this->_loaded;
	}

	/**
	 * Enter description here ...
	 * @return boolean true: success, false: anything goes wrong
	 */
	abstract public function clearPersistedState();

	public function __toString() {
		return 'S(' . $this->_jobName . ':' . $this->_jobID . '@' . $this->_jobHost . ')';
	}

	protected function getNextFreeStepID() {
		return count($this->_steps);
	}
}